% BPSOGSA code v1.0.
% Generated by Seyedali Mirjalili, 2014.

% The original codes of GSA algorithm was used in this file as
% follows:

% GSA code v1.0.
% Generated by Esmat Rashedi, 2009. 
% Adopted from: "	E. Rashedi, H. Nezamabadi-pour and S. Saryazdi,
% GSA: A Gravitational Search Algorithm, Information sciences, vol. 179,
% no. 13, pp. 2232-2248, 2009."

% Gravitational Search Algorithm.
function [Fbest,Lbest,BestChart,MeanChart]=BPSOGSA(N,noV, Max_iteration,ElitistCheck,CostFunction)

%V:   Velocity.
%a:   Acceleration.
%M:   Mass.  Ma=Mp=Mi=M;
%dim: Dimension of test function.
%N:   Number of agents.
%X:   Position of agents. dim-by-N matrix.
%R:   Distance between agents in search space.
%[low-up]: Allowable range for search space.
%Rnorm:  Norm in eq.8.
%Rpower: Power of R in eq.7.
 
 Rnorm=2; 
 Rpower=1; 
 min_flag=1; % 1: minimization, 0: maximization
 
%get allowable range and dimension of the test function.
dim=noV;

%random initialization for agents.
X=initialization(dim,N); 

%create chart of best so far and average fitnesses.
BestChart=[];MeanChart=[];

V=zeros(dim,N);

for iteration=1:Max_iteration

    %Evaluation of agents. 
    fitness=evaluateF(X,CostFunction); 
    
    [best best_X]=min(fitness); %min: minimization.  max: maximization.
    
    if iteration==1
       Fbest=best;Lbest=X(:,best_X);
    end
    if best<Fbest  % < : minimization. > : maximization
       Fbest=best;Lbest=X(:,best_X);
    end
      
BestChart=[BestChart Fbest];
MeanChart=[MeanChart mean(fitness)];

%Calculation of M
[M]=massCalculation(fitness,min_flag); 

%Calculation of Gravitational constant.
G=Gconstant(iteration,Max_iteration); 

%Calculation of accelaration in gravitational field.
a=Gfield(M,X,G,Rnorm,Rpower,ElitistCheck,iteration,Max_iteration);

%Agent movement.
[X,V]=move(X,a,V,iteration,Max_iteration,Lbest,N,dim);


end 

